/***************************************************************************
                          plugin_katesymbolviewer.cpp  -  description
                             -------------------
    begin                : Apr 2 2003
    author               : 2003 Massimo Callegari
    email                : massimocallegari@yahoo.it
    
    Changes:
    Apr 07 2003 v 0.1 - First version.
    Apr 19 2003 v 0.2 - Added to CVS. Extract functions and structures
    Apr 22 2003 v 0.3 - Added macro extraction + several bugfixing
    Apr 23 2003 v 0.4 - "View Symbol" moved in Settings menu. "Refresh List" is no
                        longer in Kate menu. Moved into a popup menu activated by a
                        mouse right button click. + Bugfixing.
    Apr 24 2003 v 0.5 - Added three check buttons in popup menu to show/hide symbols
    May 04 2003 v 0.6 - Symbol List becomes a KListView object. Removed Tooltip class.
                        Added a QTimer that every 200ms checks:
                        * if the list width has changed
                        * if the document has changed
                        Added an entry in the popup menu to switch between List and Tree mode
                        Various bugfixing.
    Jun 19 2003 v.1.0 - Removed QTimer (polling is Evil(tm)... )
                        - Captured documentChanged() event to refresh symbol list
                        - Tooltips vanished into nowhere...sigh :(
    Nov 28 2003 v.1.1 - Structured for multilanguage support
                        Added preliminary Tcl/Tk parser (thanks Rohit). To be improved.
                        Various bugfixing.
 ***************************************************************************/
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 
#include "plugin_katesymbolviewer.h"
#include "plugin_katesymbolviewer.moc"

#include <kaction.h>
#include <klocale.h>
#include <kstandarddirs.h>

class PluginView : public KXMLGUIClient
{             
  friend class KatePluginSymbolViewer;

  public:
    Kate::MainWindow *win;
};

extern "C"
{
  void* init_katecppsymbolviewerplugin()
  {
    KGlobal::locale()->insertCatalogue("katesymbolviewer"); 
    return new KatePluginFactory;
  }
}

KatePluginFactory::KatePluginFactory()
{
  s_instance = new KInstance( "kate" );
}

KatePluginFactory::~KatePluginFactory()
{
  delete s_instance;
}

QObject* KatePluginFactory::createObject( QObject* parent, const char* name, const char*, const QStringList & )
{
  return new KatePluginSymbolViewer( parent, name );
}

KInstance* KatePluginFactory::s_instance = 0L;

KateSymbolList::KateSymbolList ()
{

}

KatePluginSymbolViewer::KatePluginSymbolViewer( QObject* parent, const char* name )
    : Kate::Plugin ( (Kate::Application*)parent, name )
{
 m_Active = false;
 popup = new QPopupMenu();
 popup->insertItem(i18n("Refresh List"), this, SLOT(slotRefreshSymbol()));
 popup->insertSeparator();
 m_macro = popup->insertItem(i18n("Show Macros"), this, SLOT(toggleShowMacros()));
 m_struct = popup->insertItem(i18n("Show Structures"), this, SLOT(toggleShowStructures()));
 m_func = popup->insertItem(i18n("Show Functions"), this, SLOT(toggleShowFunctions()));
 popup->insertSeparator();
 popup->insertItem(i18n("List/Tree Mode"), this, SLOT(slotChangeMode()));
 
 popup->setItemChecked(m_macro, true);
 popup->setItemChecked(m_struct, true);
 popup->setItemChecked(m_func, true);
 macro_on = true;
 struct_on = true;
 func_on = true;
 
}

void KatePluginSymbolViewer::toggleShowMacros(void)
{
 bool s = !popup->isItemChecked(m_macro);
 popup->setItemChecked(m_macro, s);
 macro_on = s;
 slotRefreshSymbol();
}

void KatePluginSymbolViewer::toggleShowStructures(void)
{
 bool s = !popup->isItemChecked(m_struct);
 popup->setItemChecked(m_struct, s);
 struct_on = s;
 slotRefreshSymbol();
}

void KatePluginSymbolViewer::toggleShowFunctions(void)
{
 bool s = !popup->isItemChecked(m_func);
 popup->setItemChecked(m_func, s);
 func_on = s;
 slotRefreshSymbol();
}

KatePluginSymbolViewer::~KatePluginSymbolViewer()
{
}

void KatePluginSymbolViewer::addView(Kate::MainWindow *win)
{
    PluginView *view = new PluginView ();
             
     (void) new KToggleAction ( i18n("Show Symbols"), 0, this,
                      SLOT( slotInsertSymbol() ), view->actionCollection(),
                      "view_insert_symbolviewer" );
                      
    view->setInstance (new KInstance("kate"));
    view->setXMLFile("plugins/katecppsymbolviewer/ui.rc");
    win->guiFactory()->addClient (view);
    view->win = win; 
    
   m_views.append (view);
}   

void KatePluginSymbolViewer::removeView(Kate::MainWindow *win)
{
  for (uint z=0; z < m_views.count(); z++)
    if (m_views.at(z)->win == win)
    {
      PluginView *view = m_views.at(z);
      m_views.remove (view);
      win->guiFactory()->removeClient (view);
      delete view;
    }  
}

void KatePluginSymbolViewer::slotInsertSymbol()
{
 QPixmap cls( ( const char** ) class_xpm );
 Kate::ToolViewManager *kt = application()->activeMainWindow()->toolViewManager();
 Kate::DocumentManager *kd = application()->documentManager();
 
 if (m_Active == false)
     {
      symbols = new KateSymbolList();
      listMode = 0;
      
      connect(symbols, SIGNAL(executed(QListViewItem *)), this, SLOT(goToSymbol(QListViewItem *)));
      connect(symbols, SIGNAL(rightButtonClicked(QListViewItem *, const QPoint&, int)),
               SLOT(slotShowContextMenu(QListViewItem *, const QPoint&, int)));
      connect(kd, SIGNAL(documentChanged()), this, SLOT(slotDocChanged()));
      connect(symbols, SIGNAL(resizeEvent(QResizeEvent * e)), this, SLOT(slotViewChanged(QResizeEvent * e)));
      
      if (kt)
          kt->addToolViewWidget(KDockWidget::DockLeft, symbols, cls, i18n("Symbol List"));
          
      m_Active = true;
      //symbols->addColumn(i18n("Symbols"), symbols->parentWidget()->width());
      symbols->addColumn(i18n("Symbols"));
      symbols->setSorting(-1, FALSE);
      symbols->setRootIsDecorated(0);
      symbols->setTreeStepSize(10);
      symbols->setShowToolTips(TRUE);
     
      /* First Symbols parsing here...*/
      parseSymbols();
     }
  else
     {
      if (kt)
        kt->removeToolViewWidget(symbols);
      m_Active = false;
     }
}

void KatePluginSymbolViewer::slotRefreshSymbol()
{
 symbols->clear();
 parseSymbols();
}

void KatePluginSymbolViewer::slotChangeMode()
{
 listMode = !listMode;
 symbols->clear();
 parseSymbols();
}

void KatePluginSymbolViewer::slotDocChanged()
{
 //kdDebug(13000)<<"Document changed !!!!"<<endl;
 slotRefreshSymbol();
}

void KatePluginSymbolViewer::slotViewChanged(QResizeEvent * e)
{
 kdDebug(13000)<<"View changed !!!!"<<endl;
 symbols->setColumnWidth(0, symbols->parentWidget()->width());
}

void KatePluginSymbolViewer::slotShowContextMenu(QListViewItem *it, const QPoint &p, int col)
{
 popup->popup(p);
}

void KatePluginSymbolViewer::parseSymbols(void)
{
 Kate::Document *kv = application()->activeMainWindow()->viewManager()->activeView()->getDoc();
 QString docName = kv->docName();
 if(docName.endsWith(".cpp") || docName.endsWith(".c") || docName.endsWith(".h"))
     parseCppSymbols();
 else if(docName.endsWith(".tcl") || docName.endsWith(".tk")) 
     parseTclSymbols();
}

void KatePluginSymbolViewer::goToSymbol(QListViewItem *it)
{
 Kate::View *kv = application()->activeMainWindow()->viewManager()->activeView();
 kdDebug(13000)<<"Slot Activated at pos: "<<symbols->itemIndex(it) <<endl;
 kv->gotoLineNumber(positions.at(symbols->itemIndex(it)) + 10);
 kv->setFocus();
 kv->gotoLineNumber(positions.at(symbols->itemIndex(it)));
}



